﻿# -*- coding: utf-8 -*-

__version__ = 'V1.0 P2.7 W0.9.7 16.05.2015'
__author__  = 'StranikS_Scan'

'''

Тест соответствия разброса на сервере с разбросом, получаемым по формулам клиента.

Данные с сервера (период 0.1 сек без учета сети и загруженности сервера):
 [] - время с начала текущего этапа боя
 R's - ВМР орудия на дистанции 100 метров
 Кэs*Кпохs, Квбs, Кдхs, Кпхs - коэф. разброса, см. тему
 T0s - постоянная времени сведения орудия c учетом прокачки экипажа

Данные из xml клиента:
 A0 - величина тангенса угла разброса орудия (м/м)
 Кв0, Кв1, Кв2, Кпо0, Кпо1, Км - коэф. разброса, см. тему
 
Данные, рассчитываемые клиентом:
 R' - ВМР орудия на дистанции 100 метров
 KR' - коэффициент мгновенного ВМР
 V - мгновенная скорость движения танка по прямой
 W - мгновенная угловая скорость поворота танка    
 Wt - мгновенная угловая скорость поворота башни

'''

import BigWorld
import ResMgr
import BattleReplay
from Avatar import PlayerAvatar
from gui.Scaleform.Battle import Battle
from datetime import datetime

# LOG -------------------------------------------------------------------

LOG = None
LOG_PATH_DEF = ''

def getRoot():
    root = ''
    values = ResMgr.openSection('../paths.xml')['Paths'].values()[0:2]
    for value in values:
        root = value.asString + '/scripts/client/mods/'
        break
    return root

def getCurrentTime():
    replayCtrl = BattleReplay.g_replayCtrl
    arena = BigWorld.player().arena
    remainingTime = replayCtrl.getArenaLength() if replayCtrl.isPlaying else arena.periodEndTime - BigWorld.serverTime()
    currentTime = arena.periodLength - remainingTime
    return currentTime

def add_onArenaVehicleKilled(targetID, atackerID, reason):
    global LOG
    if targetID == BigWorld.player().playerVehicleID:
        if LOG is not None:
            LOG.close()
            LOG = None
    return None

def new_BattleafterCreate(self):
    global LOG
    if old_BattleafterCreate is not None:
        old_BattleafterCreate(self)
    BigWorld.player().arena.onVehicleKilled += add_onArenaVehicleKilled
    LOG = open((LOG_PATH_DEF if LOG_PATH_DEF else getRoot()) + 'DisAngleTest_' + datetime.now().strftime('%d%m%y_%H%M%S_%f')[:17] + '.log','w')
    # MOD *******************************************************************
    BigWorld.player().enableServerAim(True)    

def new_BattlebeforeDelete(self):
    global LOG
    if LOG is not None:
        LOG.close()
        LOG = None
    BigWorld.player().arena.onVehicleKilled -= add_onArenaVehicleKilled
    if old_BattlebeforeDelete is not None:
        old_BattlebeforeDelete(self)

old_BattleafterCreate = Battle.afterCreate
Battle.afterCreate = new_BattleafterCreate

old_BattlebeforeDelete = Battle.beforeDelete
Battle.beforeDelete = new_BattlebeforeDelete

# MOD *******************************************************************

def new_enableServerAim(self, enable):
    if not enable:
        enable = True
    if old_enableServerAim is not None:
        old_enableServerAim(self, enable)

def new_PlayerAvatar_updateGunMarker(self, shotPos, shotVec, dispersionAngle):
    LOG.write("[%.9f] R's:\t%.12f\n" % (getCurrentTime(), dispersionAngle*100))
    if old_PlayerAvatar_updateGunMarker is not None:
        old_PlayerAvatar_updateGunMarker(self, shotPos, shotVec, dispersionAngle)

def new_PlayerAvatar_updateTargetingInfo(self, turretYaw, gunPitch, maxTurretRotationSpeed, maxGunRotationSpeed, shotDispMultiplierFactor, \
                  gunShotDispersionFactorsTurretRotation, chassisShotDispersionFactorsMovement, chassisShotDispersionFactorsRotation, aimingTime):
    descr = self.vehicleTypeDescriptor
    LOG.write('[%.9f] Кэs*Кпохs:\t%.12f\tКвбs:\t%.12f\tКдхs:\t%.12f\tКпхs:\t%.12f\tТ0s:\t%.12f\tA0:\t%.12f\tКв0:\t%.12f\tКв1:\t%.12f\tКв2:\t%.12f\tКпо0:\t%.12f\tКпо1:\t%.12f\tКм:\t%.12f\n' % (getCurrentTime(), shotDispMultiplierFactor, \
                gunShotDispersionFactorsTurretRotation, chassisShotDispersionFactorsMovement, chassisShotDispersionFactorsRotation, aimingTime, descr.gun['shotDispersionAngle'], \
                0, descr.gun['shotDispersionFactors']['afterShot'] if 'afterShot' in descr.gun['shotDispersionFactors'] else 0, \
                descr.gun['shotDispersionFactors']['afterShotInBurst'] if 'afterShotInBurst' in descr.gun['shotDispersionFactors'] else 0, \
                0, descr.gun['shotDispersionFactors']['whileGunDamaged'], descr.miscAttrs['additiveShotDispersionFactor']))
    if old_PlayerAvatar_updateTargetingInfo is not None:
        old_PlayerAvatar_updateTargetingInfo(self, turretYaw, gunPitch, maxTurretRotationSpeed, maxGunRotationSpeed, shotDispMultiplierFactor, \
                                             gunShotDispersionFactorsTurretRotation, chassisShotDispersionFactorsMovement, chassisShotDispersionFactorsRotation, aimingTime)

def new_PlayerAvatar_getOwnVehicleShotDispersionAngle(self, turretRotationSpeed, withShot = 0):
    result = old_PlayerAvatar_getOwnVehicleShotDispersionAngle(self, turretRotationSpeed, withShot = 0)
    try:
        descr = self.vehicleTypeDescriptor
        if withShot > 1:
            withShot = 2
        vehicleSpeed, vehicleRSpeed = self.getOwnVehicleSpeeds(True)
        LOG.write("[%.9f] R':\t%.12f\tKR':\t%.12f\tКвх:\tКв%d\tV:\t%.12f\tW:\t%.12f\tWt:\t%.12f\n" % (getCurrentTime(), result*100, float(result)/descr.gun['shotDispersionAngle'],\
                      withShot, vehicleSpeed, vehicleRSpeed, turretRotationSpeed))
    finally:
        return result

old_enableServerAim = PlayerAvatar.enableServerAim
PlayerAvatar.enableServerAim = new_enableServerAim

old_PlayerAvatar_updateGunMarker = PlayerAvatar.updateGunMarker
PlayerAvatar.updateGunMarker = new_PlayerAvatar_updateGunMarker

old_PlayerAvatar_updateTargetingInfo = PlayerAvatar.updateTargetingInfo
PlayerAvatar.updateTargetingInfo = new_PlayerAvatar_updateTargetingInfo

old_PlayerAvatar_getOwnVehicleShotDispersionAngle = PlayerAvatar.getOwnVehicleShotDispersionAngle
PlayerAvatar.getOwnVehicleShotDispersionAngle = new_PlayerAvatar_getOwnVehicleShotDispersionAngle